from __future__ import annotations

from .generic_gpu import GenericGpu
from ..ast import PsAstNode
from ..ast.expressions import PsExpression, PsLiteralExpr
from ..ast.structural import PsStructuralNode
from ..exceptions import MaterializationError
from ..functions import NumericLimitsFunctions
from ..literals import PsLiteral
from ...sympyextensions import ReductionOp
from ...types import PsType, PsIeeeFloatType


class HipPlatform(GenericGpu):
    """Platform for the HIP GPU target."""

    @property
    def required_headers(self) -> set[str]:
        return super().required_headers | {'"pystencils_runtime/hip.h"', "<limits>"}

    def resolve_numeric_limits(
        self, func: NumericLimitsFunctions, dtype: PsType
    ) -> PsExpression:
        assert isinstance(dtype, PsIeeeFloatType)

        return PsLiteralExpr(
            PsLiteral(
                f"std::numeric_limits<{dtype.c_string()}>::{func.function_name}()",
                dtype,
            )
        )

    def resolve_reduction(
        self,
        ptr_expr: PsExpression,
        symbol_expr: PsExpression,
        reduction_op: ReductionOp,
    ) -> tuple[tuple[PsStructuralNode, ...], PsAstNode]:

        raise MaterializationError("Reductions are yet not supported in HIP backend.")
