from .backend.ast.expressions import PsExpression, PsCall, PsAdd, PsSub, PsMul, PsDiv
from .backend.exceptions import FreezeError
from .backend.functions import PsMathFunction, MathFunctions
from .sympyextensions.reduction import ReductionOp

_available_operator_interface: set[ReductionOp] = {
    ReductionOp.Add,
    ReductionOp.Sub,
    ReductionOp.Mul,
    ReductionOp.Div,
}


def reduction_op_to_expr(op: ReductionOp, op1, op2) -> PsExpression:
    if op in _available_operator_interface:
        match op:
            case ReductionOp.Add:
                operator = PsAdd
            case ReductionOp.Sub:
                operator = PsSub
            case ReductionOp.Mul:
                operator = PsMul
            case ReductionOp.Div:
                operator = PsDiv
            case _:
                raise FreezeError(
                    f"Found unsupported operation type for reduction assignments: {op}."
                )
        return operator(op1, op2)
    else:
        match op:
            case ReductionOp.Min:
                return PsCall(PsMathFunction(MathFunctions.Min), [op1, op2])
            case ReductionOp.Max:
                return PsCall(PsMathFunction(MathFunctions.Max), [op1, op2])
            case _:
                raise FreezeError(
                    f"Found unsupported operation type for reduction assignments: {op}."
                )
